library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.numeric_std.all;

entity PFX_MultiBoot_Module is

    port (	TRIGGER: in    std_logic;
				SYSCLK : in std_logic
         );

end PFX_MultiBoot_Module;

architecture Behavioral of PFX_MultiBoot_Module is

component ICAP_VIRTEX5
  generic
  ( ICAP_WIDTH : string := "X32"
  );
  port
  (
    CE       : in  std_ulogic;                      -- Clock enable input
    CLK      : in  std_ulogic;                      -- Clock input
    I        : in  std_logic_vector (31 downto 0);  -- 32-bit data input
    WRITE    : in std_ulogic                        -- Write input
  );
end component;

type CTRL_STATE is (STATE_00, STATE_01, STATE_02, STATE_03, STATE_04, STATE_05,
   STATE_06, STATE_07, STATE_08, STATE_09, STATE_10, STATE_11);
signal NEXT_STATE    : CTRL_STATE := STATE_00;

signal CE       : std_logic := '1';
signal I        : std_logic_vector(31 downto 0) := "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
signal ICAP_WRITE    : std_logic := '1';

begin

ICAP_VIRTEX5_inst : ICAP_VIRTEX5
generic map (ICAP_WIDTH => "X32") -- "X8" or "X32
port map (
  CE       => CE,        	 -- Clock enable input
  CLK      => SYSCLK,       -- Clock input
  I        => I,         	 -- 32-bit data input
  WRITE    => ICAP_WRITE    -- Write input
);

process(SYSCLK)
begin
	if (falling_edge(SYSCLK)) then
		if (TRIGGER = '0') then
          case NEXT_STATE is
            when STATE_00 =>
              ICAP_WRITE    <= '1';
              CE       <= '1';
              I        <= x"00000000";
              NEXT_STATE    <= STATE_01;
            when STATE_01 =>
              ICAP_WRITE    <= '0';
              CE       <= '1';
              I        <= x"00000000";
              NEXT_STATE    <= STATE_02;
            when STATE_02 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"00000000";
              NEXT_STATE    <= STATE_03;
            when STATE_03 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"FFFFFFFF";   -- dummy word
              NEXT_STATE    <= STATE_04;
            when STATE_04 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"5599AA66";   -- sync word
              NEXT_STATE    <= STATE_05;
            when STATE_05 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"04000000";   -- Type 1 NO OP
              NEXT_STATE    <= STATE_06;
            when STATE_06 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"0C400080";   -- Type 1 Write 1 word to WBSTAR
              NEXT_STATE    <= STATE_07;
            when STATE_07 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"28000000";    -- Warm boot start address
              NEXT_STATE    <= STATE_08;
            when STATE_08 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"0C000180";   -- Type 1 write 1 word to CMD
              NEXT_STATE    <= STATE_09;
            when STATE_09 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"000000F0";   -- IPROG command
              NEXT_STATE    <= STATE_10;
            when STATE_10 =>
              ICAP_WRITE    <= '0';
              CE       <= '0';
              I        <= x"04000000";   -- Type 1 NO OP
              NEXT_STATE    <= STATE_11;
            when STATE_11 =>
              ICAP_WRITE    <= '0';
              CE       <= '1';           -- deassert CE
              I        <= x"04000000";
              NEXT_STATE    <= STATE_11;
            when others  =>
              ICAP_WRITE    <= '1';
              CE       <= '1';
              I        <= x"AAAAAAAA";
              NEXT_STATE    <= STATE_00;
          end case;
        else
          ICAP_WRITE    <= '1';
          CE       <= '1';
          I        <= x"AAAABBBB";
          NEXT_STATE    <= STATE_00;
			end if;
    end if;
end process;

end Behavioral ;