LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;

ENTITY NAND_FLASH IS
  PORT(
		R_B2	  	    					: IN  STD_LOGIC;
		R_B1	  	    					: IN  STD_LOGIC;
		rst								: IN  STD_LOGIC;
		id_chk							: IN  STD_LOGIC;
		clock								: IN  STD_LOGIC;
		prog								: IN  STD_LOGIC;
		n_mode							: IN  STD_LOGIC;
		ID_OK 	  	    				: OUT STD_LOGIC;
		RE 	  	    					: OUT STD_LOGIC;
		CE1 	  	    					: OUT STD_LOGIC;
		CE2 	  	    					: OUT STD_LOGIC;
		CLE 	  	    					: OUT STD_LOGIC;
		ALE 	  	    					: OUT STD_LOGIC;
		WE 		  	  				  	: OUT STD_LOGIC;
		WP 		  	    				: OUT STD_LOGIC;
		IO		 	  	    				: INOUT  STD_LOGIC_VECTOR(7 downto 0));
END NAND_FLASH;

ARCHITECTURE logic OF NAND_FLASH IS

signal C1_C1,C1_C2,C1_C3,C1_C4,C1_C5,NAND_CE,check_cycle1,check_cycle2,check_cycle3,check_cycle4,
check_cycle5,C2_C1,C2_C2,C2_C3,C2_C4,C2_C5,stop_nand,n_count_start,read_second_chip :STD_LOGIC;

signal count_n 	:STD_LOGIC_VECTOR(24 DOWNTO 0);

BEGIN

process(id_chk,stop_nand,rst)
begin
  if (stop_nand = '1') or (rst = '0') then
      n_count_start <= '0';
  elsif (id_chk = '1') then
		n_count_start <= '1';
  end if;
end process;

process(clock,n_count_start,read_second_chip)
begin
	if (n_count_start='0') or (read_second_chip='1') then
    count_n <= (others => '0');
 elsif (clock'event and clock = '1') then 
			count_n <= count_n + '1';
	end if;
end process;

stop_nand <= '1' when count_n > X"888888" and R_B1='1' and R_B2='1' else '0';

WE <= '0' when ((count_n < X"3000") or (count_n > X"301F" and count_n < X"3033") or (count_n > X"30BD" and count_n < X"30D1") 
or (count_n > X"30FE" and count_n < X"3112")) and n_count_start='1' else '1' when prog = '0' else 'Z';

ALE <= '1' when (count_n > X"30F0" and count_n < X"3114" and n_count_start='1') else '0' when prog = '0' else 'Z';

CLE <= '1' when ((count_n > X"3011" and count_n < X"3034") or (count_n > X"30AF" and count_n < X"30D2")) and n_count_start='1' else '0' when prog = '0' else 'Z';

RE <= '0' when ((count_n > X"319A" and count_n < X"36FD") or (count_n > X"3721" and count_n < X"3C84") or (count_n > X"3CA8" and count_n < X"420B")
or (count_n > X"422F" and count_n < X"4792") or (count_n > X"47B6" and count_n < X"4D19")) and n_count_start='1' else '1' when prog = '0' else 'Z'; -- five read cycles

IO <= X"FF" when (count_n < X"30B8" and n_count_start='1') else --reset command
X"90" when (count_n > X"30B8" and count_n < X"30F9" and n_count_start='1') else --read ID command
X"00" when (count_n > X"30F9" and count_n < X"313A" and n_count_start='1') else "ZZZZZZZZ"; --address. 1cycle

read_second_chip <= '1' when count_n = X"5555" and NAND_CE = '0' and n_count_start='1' else '0';

process(read_second_chip,n_count_start)
begin
	if (n_count_start='0') then
   NAND_CE <= '0';
 elsif (read_second_chip = '1') then 
	NAND_CE <= '1';
	end if;
end process;

WP  <= '1' when prog = '0' else 'Z';
CE1 <= NAND_CE when prog = '0' else 'Z';
CE2 <= not NAND_CE when prog = '0' else 'Z';

check_cycle1 <= '1' when count_n = X"344C" and IO = X"EC" and n_count_start='1' else '0'; -- K9WAG08U1B maker code
check_cycle2 <= '1' when count_n = X"39D3" and IO = X"D3" and n_count_start='1' else '0'; -- K9WAG08U1B device code
check_cycle3 <= '1' when count_n = X"3F5A" and IO = X"51" and n_count_start='1' else '0'; -- K9WAG08U1B 3rd ID Data
check_cycle4 <= '1' when count_n = X"44E1" and IO = X"95" and n_count_start='1' else '0'; -- K9WAG08U1B 4th ID Data
check_cycle5 <= '1' when count_n = X"4A68" and IO = X"58" and n_count_start='1' else '0'; -- K9WAG08U1B 5th ID Data

process(check_cycle1,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C1_C1 <= '0'; -- chip1, read cycle 1
 elsif (check_cycle1 = '1' and NAND_CE = '0') then 
	C1_C1 <= '1';
	end if;
end process;

process(check_cycle2,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C1_C2 <= '0'; -- chip1, read cycle 2
 elsif (check_cycle2 = '1' and NAND_CE = '0') then 
	C1_C2 <= '1';
	end if;
end process;

process(check_cycle3,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C1_C3 <= '0'; -- chip1, read cycle 3
 elsif (check_cycle3 = '1' and NAND_CE = '0') then 
	C1_C3 <= '1';
	end if;
end process;

process(check_cycle4,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C1_C4 <= '0'; -- chip1, read cycle 4
 elsif (check_cycle4 = '1' and NAND_CE = '0') then 
	C1_C4 <= '1';
	end if;
end process;

process(check_cycle5,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C1_C5 <= '0'; -- chip1, read cycle 5
 elsif (check_cycle5 = '1' and NAND_CE = '0') then 
	C1_C5 <= '1';
	end if;
end process;

process(check_cycle1,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C2_C1 <= '0'; -- chip2, read cycle 1
 elsif (check_cycle1 = '1' and NAND_CE = '1') then 
	C2_C1 <= '1';
	end if;
end process;

process(check_cycle2,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C2_C2 <= '0'; -- chip2, read cycle 2
 elsif (check_cycle2 = '1' and NAND_CE = '1') then 
	C2_C2 <= '1';
	end if;
end process;

process(check_cycle3,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C2_C3 <= '0'; -- chip2, read cycle 3
 elsif (check_cycle3 = '1' and NAND_CE = '1') then 
	C2_C3 <= '1';
	end if;
end process;

process(check_cycle4,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C2_C4 <= '0'; -- chip2, read cycle 4
 elsif (check_cycle4 = '1' and NAND_CE = '1') then 
	C2_C4 <= '1';
	end if;
end process;

process(check_cycle5,n_mode,NAND_CE)
begin
	if (n_mode='0') then
   C2_C5 <= '0'; -- chip2, read cycle 5
 elsif (check_cycle5 = '1' and NAND_CE = '1') then 
	C2_C5 <= '1';
	end if;
end process;

ID_OK <= C1_C1 and C1_C2 and C1_C3 and C1_C4 and C1_C5 and C2_C1 and C2_C2 and C2_C3 and C2_C4 and C2_C5;

END logic;
